package com.jzdsh.auto.biz;

import com.jfinal.aop.Before;
import com.jfinal.aop.Inject;
import com.jfinal.kit.Ret;
import com.jfinal.plugin.activerecord.tx.Tx;
import com.jzdsh.auto.model.*;
import com.jzdsh.util.Api;
import com.jzdsh.util.CommonCons;
import com.jzdsh.util.CommonUtil;
import com.jzdsh.util.MapUtils;
import com.jzdsh.vo.HomepageVo;
import com.jzdsh.vo.UserStatus;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.lang3.StringUtils;

import java.util.*;

/**
 * @ProjectName: auto
 * @Package: com.jzdsh.auto.biz
 * @ClassName: HomepageService
 * @Author: Jzdsh
 * @Description: 主页
 * @Date: 2020/3/21 10:59
 * @Version: 1.0
 */
@Log4j2
public class HomepageService {

    @Inject
    NoticeService noticeService;

    @Inject
    UserInfoService userInfoService;

    @Inject
    ServiceCategoryService categoryService;

    @Inject
    UdaService udaService;

    @Inject
    OwnerService ownerService;

    @Inject
    ShopService shopService;

    @Inject
    CouponService couponService;

    @Inject
    SearchProviderService searchProviderService;

    @Inject
    BrandFourSService brandFourSService;

    @Before(Tx.class)
    public Ret show(String token, String longitude, String latitude, int pageNum) {
        HomepageVo vo = new HomepageVo( );

        try {
            if (StringUtils.isBlank(longitude) || Objects.equals(CommonCons.UNDEFINED, longitude)) {
                longitude = "0";
                latitude = "0";
            }

            //-- 获取当前地理信息 区域
            Map< String, String > map = MapUtils.getMap(longitude, latitude);
            String province = map.get("province");
            String city = map.get("city");
            String district = map.get("district");
            if (CommonCons.BRACKETS.equals(city)) {
                city = province;
            }
            //城市
            vo.setDistrict(district);
            //存在token情况下
            haveToken(vo, token, longitude, latitude, province, city, district);
            //层级
            haveLevel(vo);
            //--车主服务
            vo.setOwner(ownerService.getByIsRecommend( ));
            //--

            //商家信息
            List< Shop > shop = shopService.findBySearchName(longitude, latitude, pageNum, CommonCons.PAGE_SIZE);

            shop.forEach(sp -> {
                String pMainBusiness = sp.getPMainBusiness( );
                if (pMainBusiness != null) {
                    List< String > l = Arrays.asList(pMainBusiness.split(","));
                    if (l.size( ) > 3) {
                        List< String > list = l.subList(0, 3);
                        sp.set("pMainBusiness", list);
                    } else {
                        sp.set("pMainBusiness", l);
                    }
                }

                String format = String.format("%.0f", sp.getDouble("distance") * 1000);
                Float distance = Float.valueOf(format);
                if (distance >= 1000) {
                    sp.put("distance", "距您" + String.format("%.1f", distance / 1000) + "km");
                } else {
                    sp.put("distance", "距您" + String.format("%.0f", distance) + "m");
                }
            });
            vo.setShop(shop);
            return Ret.ok("data", Api.ok(vo));
        } catch (Exception e) {
            log.info("e", e);
        }
        return Ret.fail("data", Api.fail(null));
    }

    private void haveLevel(HomepageVo vo) {
        //层级
        List< ServiceCategory > levelOne = categoryService.getLeve(CommonCons.ROLE_ONE, CommonCons.LEVEL_ONE);
        vo.setServiceCategoryOne(levelOne);
        List< ServiceCategory > levelOnes = new ArrayList<>( );
        levelOnes.addAll(levelOne);
        ServiceCategory serviceCategory = new ServiceCategory( );
        serviceCategory.setId(0);
        serviceCategory.setName("推荐");
        serviceCategory.setPId(0L);
        levelOnes.add(0, serviceCategory);
        vo.setServiceCategoryOnes(levelOnes);
        //第二层
        List< ServiceCategory > levelTwo = categoryService.getLevelShow(CommonCons.ROLE_ONE, CommonCons.LEVEL_TWO, 9);
        levelTwo.forEach(o -> {
            o.put("ids", "y" + o.getId( ));

        });
        vo.setServiceCategoryTwo(levelTwo);
        //
    }


    private void haveToken(HomepageVo vo, String token, String longitude, String latitude, String province, String city, String district) {
        try {
            if (StringUtils.isNotBlank(token)) {
                UserInfo userInfo = userInfoService.findByToken(token);
                userInfo.setLongitude(longitude);
                userInfo.setLatitude(latitude);
                userInfo.update( );
                //返回用户 认证信息
                vo.setRoleType(userInfo.getRole( ));
                //返回用户认证状态
                UserStatus userStatus = userInfoService.getUserStatus(userInfo.getId( ));
                vo.setUserStatus(userStatus);
                //end
                Uda udaInfo;
                udaInfo = udaService.getByUserId(userInfo.getId( ));
                if (udaInfo == null) {
                    //证明是第一次进去该系统
                    Uda uda = new Uda( );
                    uda.setUserId(userInfo.getId( ));
                    uda.setProvince(province);
                    uda.setCity(city);
                    uda.setDistrict(district);
                    uda.save( );
                } else {
                    province = udaInfo.getProvince( );
                    city = udaInfo.getCity( );
                    district = udaInfo.getDistrict( );
                }
                // 用户修改
                {
                    HomepageVo.InnerCity innerCity = vo.new InnerCity( );
                    if (udaInfo.getCity( ).equals(city) || city.equals(userInfo.getNewCity( ))) {
                        innerCity.setBoolen(true);
                        innerCity.setNewCity(city);
                        innerCity.setOldCity(udaInfo.getCity( ));
                    } else {
                        innerCity.setBoolen(false);
                        innerCity.setNewCity(city);
                        innerCity.setOldCity(udaInfo.getCity( ));
                        innerCity.setTips("定位到您在 ♢" + city + "是否切换到该城市进行探索?");
                    }
                    vo.setInnerCity(innerCity);
                }
            }
            //---公告
            List< Notice > notices = noticeService.listByProvinceCityPageSize(province, city, 10);
            vo.setNotice(notices);
        } catch (Exception e) {
            log.info("子类show" + e);
        }
    }

    /**
     * 免费入住
     *
     * @param token
     * @param role
     * @param coName
     * @param name
     * @param city
     * @param street
     * @param mainBusiness
     * @param pMainBusiness
     * @param introduce
     * @param authPhone
     * @param referrerPhone
     * @return
     */
    public Ret save(String token, String role, String coName, String name, String city, String street, String houseNumber, String mainBusiness, String pMainBusiness, String introduce, String authPhone, String referrerPhone, String paramType) {

        Shop shop = new Shop( );
        if (StringUtils.isBlank(role)) {
            role = CommonCons.ROLE_ONE + "";
        }

        UserInfo userInfo = userInfoService.findByToken(token);
        userInfo.setRole(role);
        userInfo.update( );

        Shop reShop = shopService.getByUserId(userInfo.getId( ));
        if (reShop != null) {
            reShop.update( );
            String str = "亲~ 您已获得优惠券!";
            return Ret.fail("date", Api.fail(str));
        }

        //lc 20191018 增加优惠券
        couponService.saveCommonAndSharerCoupon(userInfo.getId( ), token);


        //去掉推荐人手机号 的所有空格
        if (StringUtils.isNotBlank(referrerPhone)) {
            referrerPhone = referrerPhone.replace(" ", "");
        }
        shop.setCode(getShopCode( ));
        shop.setRole(role);
        shop.setCoName(coName);
        shop.setName(name);
        shop.setMainBusiness(mainBusiness);
        shop.setPMainBusiness(pMainBusiness);
        shop.setIntroduce(introduce);
        shop.setAuthPhone(authPhone.trim( ));
        shop.setReferrerPhone(referrerPhone);
        //判断省市县不等空
        String address = "";
        if (StringUtils.isNotBlank(city)) {
            String[] split = city.split(",+");
            shop.setProvince(split[0]);
            shop.setCity(split[1]);
            shop.setDistrict(split[2]);
            //总地址 =  省+ 市 + 县 + 街道 + 门牌号
            address = city.replace(",", "") + street + houseNumber;
            String longitudeAndLatitude = MapUtils.getLongitudeLatitude(city.replace(",", "") + street);

            //根据地址 算出经纬度
            if (StringUtils.isNotBlank(longitudeAndLatitude)) {
                String[] splits = longitudeAndLatitude.split(",+");
                shop.setLongitude(splits[0]);
                shop.setLatitude(splits[1]);
            } else {
                shop.setLongitude(0 + "");
                shop.setLatitude(0 + "");
            }
        }
        shop.setStreet(street);
        shop.setAddress(address);
        shop.setHouseNum(houseNumber);
        //是否上架  默认是上架的
        shop.setIsShelf(CommonCons.ONE);
        //入驻时间
        shop.setEnterTime(new Date( ));
        //认证状态 0未认证 1认证
        shop.setStatus(CommonCons.PROVIDER_STATUS_ZERO);
        shop.setCreateTime(new Date( ));
        shop.setRole(role);
        shop.setIsHot(CommonCons.PROVIDER_IS_HOT_ZERO);
        shop.setIsUse(CommonCons.PROVIDER_IS_USE_ONE);
        shop.setIsLine(CommonCons.ZERO);

        // 一些默认值
        shop.setStartWeek("周一");
        shop.setEndWeek("周五");
        shop.setStartTime("09:00");
        shop.setEndTime("17:00");

        if (StringUtils.isNotBlank(userInfo.getImg( ))) {
            //微信头像
            shop.setLogo(CommonUtil.wxImgSave(userInfo.getImg( )));
        } else {
            //默认logo图
            shop.setLogo(CommonCons.DEFAULT_STORE_LOGO);
        }
        //默认banner
        shop.setBanner(CommonCons.DEFAULT_STORE_BANNER);

        shop.setParamType(paramType);


        //添加 搜索表数据
        boolean roleT = CommonCons.ROLE_TWO.equals(role) && ("1".equals(paramType) || "3".equals(paramType) || "4".equals(paramType) || "5".equals(paramType));

        if (roleT) {
            ArrayList< String > pmList = new ArrayList<>( );
            if (StringUtils.isNotBlank(shop.getPMainBusiness( ))) {
                String[] split = shop.getPMainBusiness( ).split(",");
                for (String pmName : split) {
                    ServiceCategory pmObj = categoryService.getByNameRole(pmName, CommonCons.ROLE_TWO);
                    pmList.add(pmObj.getId( ) + "");
                }
            }

            String[] split = shop.getMainBusiness( ).split(",");
            String sb = "";
            for (int i = 0; i < pmList.size( ); i++) {
                for (String s : split) {
                    if (s.startsWith(pmList.get(i))) {
                        sb += (s.substring(pmList.get(i).length( ))) + ",";
                    }
                }
                sb = sb.substring(0, sb.lastIndexOf(","));
                sb += ";";
            }
            shop.setMainBusiness(sb);

        }


        if (Objects.equals(CommonCons.ROLE_ONE, role)) {
            if (Objects.equals("1", paramType)) {
                //企业类型 （以下都是默认值）
                shop.setCoType("快修连锁店");
                // 经营资质
                shop.setIntelligence("三类");
                //主营品牌
                shop.setMainBrand("各种车型");
            } else if (Objects.equals("2", paramType)) {
                //企业类型
                shop.setCoType("综合修理厂");
                // 经营资质
                shop.setIntelligence("二类");
            } else if (Objects.equals("3", paramType)) {
                //企业类型
                shop.setCoType("品牌4S店");
                // 经营资质
                shop.setIntelligence("二类");
                //主营品牌
                shop.setMainBrand("专营车型");
            }
        }
        //默认是前台加入
        shop.setIsImport(CommonCons.ZERO);

        shop.save( );


        //添加公告信息
        noticeService.save(shop.getId( ), shop.getName( ), shop.getProvince( ), shop.getCity( ), shop.getDistrict( ));


        threadExec(shop.getRole( ), shop.getId( ), paramType, shop.getPMainBusiness( ), shop.getMainBusiness( ));


        return Ret.fail("data", Api.fail(null));
    }


    private void threadExec(String roleType, int shopId, String paramType, String pMainBusiness, String mainBusiness) {
        try {
            //服务商的时候
            if (Objects.equals(CommonCons.ROLE_ONE, roleType) && StringUtils.isNotBlank(mainBusiness)) {
                new Thread(( ) -> {
                    // 1 代表 小型汽车 ,2服务商用车专修
                    if ("1".equals(paramType) || "2".equals(paramType)) {
                        smallCarSave(shopId, pMainBusiness, mainBusiness, paramType);
                    }
                    // 4s 店
                    if ("3".equals(paramType)) {
                        fourS_save(shopId, mainBusiness, pMainBusiness);
                    }
                }).start( );
            }


            //当是经销商的时候   添加经销商搜索
            if (Objects.equals(CommonCons.ROLE_TWO, roleType) && StringUtils.isNotBlank(mainBusiness)) {
                new Thread(( ) -> {
                    // 1,3,4,5 代表是 普通类型 1汽配(单项件),2汽配(全车件) ,3用品改装,4汽保设备与工具,5商用车(单项件) 6商用车(全车件)
/*                    if ("1".equals(paramType) || "3".equals(paramType) || "4".equals(paramType) || "5".equals(paramType)) {
                        saveSearchDealer(paramType, shopId, pMainBusiness, mainBusiness);
                    }
                    //  2 代表全车件
                    if ("2".equals(paramType)) {
                        saveAllCar(StoreId, pMainBusiness, mainBusiness);
                    }
                    // 6 代表商务车
                    if ("6".equals(paramType)) {
                        saveCommercial(StoreId, pMainBusiness, mainBusiness);
                    }*/
                }).start( );
            }

        } catch (Exception e) {
            e.printStackTrace( );
        }
        return;
    }

    /**
     * 小型车 商用车专修
     *
     * @param shopId
     * @param pMainBusiness
     * @param mainBusiness
     * @param paramType
     */
    private void smallCarSave(int shopId, String pMainBusiness, String mainBusiness, String paramType) {

        if (StringUtils.isBlank(pMainBusiness) || StringUtils.isBlank(mainBusiness)) {
            return;
        }
        try {
            Set< SearchProvider > spList = new HashSet<>( );
            String[] pMains = pMainBusiness.split(",");
            String[] mains = mainBusiness.split(",");
            List< String > mainList = Arrays.asList(mains);

            if (pMains.length > 0) {
                for (String main : pMains) {
                    int role = CommonCons.ROLE_ONE;
                    if ("道路救援".equals(main)) {
                        role = 3;
                    }
                    ServiceCategory category = categoryService.getByNameRole(main, role);
                    if (category != null) {
                        List< String > namesList = categoryService.getByPid(category.getId( ));
                        if (mainList != null && mainList.size( ) > 0) {
                            for (String name : mainList) {
                                if (namesList.contains(name)) {
                                    SearchProvider sp = new SearchProvider( );
                                    sp.setLevel("1");
                                    sp.setPName(main);
                                    sp.setName(name);
                                    sp.setShopId(shopId);
                                    sp.setType(paramType);
                                    spList.add(sp);
                                }
                            }
                        }
                    }

                    {
                        SearchProvider sp = new SearchProvider( );
                        sp.setLevel("2");
                        sp.setPName(main);
                        sp.setName(main);
                        sp.setShopId(shopId);
                        sp.setType(paramType);
                        spList.add(sp);
                    }
                    {
                        if (Objects.equals("2", paramType)) {
                            SearchProvider sp = new SearchProvider( );
                            sp.setLevel("3");
                            sp.setPName("商用车专修");
                            sp.setName("商用车专修");
                            sp.setShopId(shopId);
                            sp.setType(paramType);
                            spList.add(sp);
                        }
                    }
                }
            }
            List< SearchProvider > listSet = new ArrayList<>( );
            listSet.addAll(spList);
            // 有数据可添加
            if (listSet.size( ) > 0) {
                //先删除
                searchProviderService.deleteByShopId(shopId);
                searchProviderService.saveList(listSet);
            }
        } catch (Exception e) {
            log.info("", e);
        }
    }

    public void fourS_save(int shopId, String brand, String mainBusiness) {
        if (StringUtils.isBlank(mainBusiness) || StringUtils.isBlank(brand)) {
            return;
        }
        String[] split = mainBusiness.split(",");
        String[] split1 = brand.split(",");
        Set< SearchProvider > spList = new HashSet<>( );

        int i = 0;
        for (String factory : split) {
            BrandFourS brandFourS = brandFourSService.getByBrandFactory(split1[i], factory);
            if (brandFourS != null) {
                SearchProvider sp = new SearchProvider( );
                sp.setLevel("1");
                sp.setPName(brandFourS.getBrand( ));
                sp.setName(brandFourS.getFactory( ));
                sp.setShopId(shopId);
                sp.setType("3");
                spList.add(sp);

                SearchProvider sp1 = new SearchProvider( );
                sp1.setLevel("2");
                sp1.setPName(brandFourS.getBrand( ));
                sp1.setName(brandFourS.getBrand( ));
                sp1.setShopId(shopId);
                sp1.setType("3");
                spList.add(sp1);

                SearchProvider sp2 = new SearchProvider( );
                sp2.setLevel("3");
                sp2.setPName(brandFourS.getBrand( ));
                sp2.setName(brandFourS.getCategory( ));
                sp2.setShopId(shopId);
                sp2.setType("3");
                spList.add(sp2);

                SearchProvider sp3 = new SearchProvider( );
                sp3.setLevel("4");
                sp3.setPName(brandFourS.getBrand( ));
                sp3.setName("4S店");
                sp3.setShopId(shopId);
                sp3.setType("3");
                spList.add(sp3);
            }
            i++;
        }

        List< SearchProvider > listSet = new ArrayList<>( );
        listSet.addAll(spList);
        if (listSet.size( ) > 0) {
            searchProviderService.deleteByShopId(shopId);
            searchProviderService.saveList(listSet);
        }
    }


    public void saveCommercial(int shopId, String pMainBusiness, String mainBusiness) {


    }


    /**
     * 获取唯一店铺code
     *
     * @return
     */
    public String getShopCode( ) {
        String code = CommonUtil.getRandom(8);
        Shop shop = Shop.dao.findFirst("select * from shop where code = ?", code);
        if (shop == null) {
            return code;
        } else {
            return getShopCode( );
        }
    }

}
